# -*- coding=utf-8 -*-

__author__ = 'flanker'

from django.conf.urls import patterns

urlpatterns = patterns('',
                       (r'^p/$', 'moneta.views.public_check'),
                       (r'^p/get/(?P<eid>\d+)/(?P<name>.*)$', 'moneta.views.get_file_p'),
                       (r'^p/check/(?P<eid>\d+)/(?P<value>sha1|sha256|md5)$', 'moneta.views.get_checksum_p'),
                       (r'^p/signature/(?P<eid>\d+)/(?P<sid>\d+)/$', 'moneta.views.get_signature_p'),
                       (r'^a/get/(?P<eid>\d+)/(?P<name>.*)$', 'moneta.views.get_file'),
                       (r'^a/check/(?P<eid>\d+)/(?P<value>sha1|sha256|md5)$', 'moneta.views.get_checksum'),
                       (r'^a/file/(?P<eid>\d+)/$', 'moneta.views.show_file'),
                       (r'^a/add_package/(?P<rid>\d+)/$', 'moneta.views.add_element'),
                       (r'^a/post/(?P<rid>\d+)/$', 'moneta.views.add_element_post'),
                       (r'^a/sign/(?P<rid>\d+)/$', 'moneta.views.add_element_signature'),
                       (r'^a/signature/(?P<eid>\d+)/(?P<sid>\d+)/$', 'moneta.views.get_signature'),
                       (r'^a/modify/(?P<rid>\d+)/$', 'moneta.views.modify_repository'),
                       (r'^a/delete/(?P<rid>\d+)/$', 'moneta.views.delete_repository'),
                       (r'^a/delete/(?P<rid>\d+)/(?P<eid>\d+)/$', 'moneta.views.delete_element'),
                       (r'^a/search/(?P<rid>\d+)/$', 'moneta.views.search_package'),
                       (r'^a/compare/(?P<rid>\d+)/$', 'moneta.views.compare_states'),
                       (r'^a/index/$', 'moneta.views.index'),
                       (r'^a/$', 'moneta.views.private_check'),
                       (r'^a/check/$', 'moneta.views.check'),
                       (r'^test/$', 'moneta.views.test_upload'),
                       )



# -*- coding: utf-8 -*-
import os
import tarfile
import zipfile

from django.conf import settings
from django.conf.urls import url
from django.core.files.uploadedfile import UploadedFile
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _
import requests

from moneta.exceptions import InvalidRepositoryException
from moneta.repositories.aptitude import Aptitude
from moneta.repositories.base import RepositoryModel
from moneta.repositories.pypi import PyArchive
from moneta.repository.models import ArchiveState, Element, Repository, storage
from moneta.templatetags.moneta import moneta_url
from moneta.utils import parse_control_data

__author__ = 'flanker'


class RubyGem(Aptitude):
    verbose_name = _('Gem repository for Ruby packages')
    storage_uid = '71610000-0000-0000-0000-%012d'
    archive_type = 'rubygem'

    def finish_element(self, element: Element, states: list):
        """
        Called after the .save() operations, with all states associated to this new element.
        :param element: Element
        :param states: list of ArchiveState
        """
        RepositoryModel.finish_element(self, element, states)

    def is_file_valid(self, uploaded_file: UploadedFile):
        return True

    @staticmethod
    def open_file(filename, archive_file):
        """ identify compression type and return a tuple (open compressed file, compression_type, prefix).
        If compression type is unknown, return None
        """
        endswith = filename.endswith
        compressed_file = None
        # noinspection PyBroadException
        try:
            if endswith('.tar.gz') or endswith('.tar.xz') or endswith('.tar.bz2'):
                compressed_file = tarfile.open(name='file.tar', mode='r:*', fileobj=archive_file)
                prefix = os.path.commonprefix(compressed_file.getnames())
                return PyArchive(compressed_file, 'tar', prefix)
            if endswith('.zip'):
                compressed_file = zipfile.ZipFile(archive_file)
                prefix = os.path.commonprefix(compressed_file.namelist())
                return PyArchive(compressed_file, 'zip', prefix)
            if endswith('.egg'):
                compressed_file = zipfile.ZipFile(archive_file)
                prefix = os.path.commonprefix(compressed_file.namelist())
                return PyArchive(compressed_file, 'egg', prefix)
            if endswith('.whl'):
                compressed_file = zipfile.ZipFile(archive_file)
                prefix = os.path.commonprefix(compressed_file.namelist())
                return PyArchive(compressed_file, 'whl', prefix)
        except Exception:
            if compressed_file:
                compressed_file.close()
        return None

    def update_element(self, element):
        """
        Extract some informations from element to prepare the repository
        :param element: Element to add to the repository
        :return: Unicode string containing meta-data

        ar -x control.tar.gz
        tar -xf control.tar.gz control
        """
        archive_file = storage(settings.STORAGE_ARCHIVE).get_file(element.archive_key)
        py_archive = self.open_file(element.filename, archive_file)
        if py_archive is None:
            raise InvalidRepositoryException(_('Unable to open file'))
        try:
            control_data_value = py_archive.get_pkg_info()
            if not control_data_value:
                raise InvalidRepositoryException(_('No control data in archive'))
            element.extra_data = control_data_value
            control_data = parse_control_data(control_data_value, continue_line='        ', skip_after_blank=True)
            for key, attr in (('Name', 'archive'), ('Version', 'version'), ('Home-page', 'official_link'),
                              ('Description', 'long_description')):
                if key in control_data:
                    setattr(element, attr, control_data.get(key, ''))
            element.archive = element.archive.replace('-', '').replace('_', '')
            element.name = element.archive
        finally:
            py_archive.close()
            archive_file.close()

    def public_url_list(self):
        """
        Return a list of URL patterns specific to this repository
        Sample recognized urls:
            *

        :return: a patterns as expected by django

        """
        pattern_list = [
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/specs.4.8.gz$",
                self.wrap_view('simple'), name="specs"),

            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/s/(?P<state_slug>[\w\-\._]+)/simple/?$",
                self.wrap_view('simple'), name="simple"),
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/s/(?P<state_slug>[\w\-\._]+)/simple/"
                "(?P<search_pattern>[a-z\d_\-]*)/?$", self.wrap_view('simple'),
                name="simple"),
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/s/(?P<state_slug>[\w\-\._]+)/rpc/?$",
                self.wrap_view('xmlrpc', csrf_exempt=True), name="xmlrpc"),
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/simple/?$", self.wrap_view('simple'), name="simple"),
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/simple/(?P<search_pattern>[a-zA-Z\d_\-]*)/?$",
                self.wrap_view('simple'), name="simple"),
            url(r"^\d+/[\w\-\._]+/[\w\-\._]+/f/(?P<eid>\d+)/.*$", self.wrap_view('get_filename'), name="get_file"),
            url(r"^(?P<rid>\d+)/$", self.wrap_view('index'), name="index"),
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/$", self.wrap_view('search_plugin'), name="search_plugin"),
            url(r"^(?P<rid>\d+)/(?P<repo_slug>[\w\-\._]*)/plugin/(?P<eid>\d+)$", self.wrap_view('search_plugin'), name="search_plugin"),
        ]
        return pattern_list

    def index(self, request, rid):
        repo = get_object_or_404(Repository.reader_queryset(request), id=rid, archive_type=self.archive_type)
        states = list(ArchiveState.objects.filter(repository=repo).order_by('name'))
        template_values = {'repo': repo, 'states': states, 'upload_allowed': repo.upload_allowed(request),
                           'admin_allowed': repo.admin_allowed(request), }
        view_name = moneta_url(repo, 'simple')
        tab_infos = [
            (reverse(view_name, kwargs={'rid': repo.id, 'repo_slug': repo.slug}), states,
             ArchiveState(name=_('All states'), slug='all-states')),
        ]
        for state in states:
            tab_infos.append(
                (reverse(view_name, kwargs={'rid': repo.id, 'repo_slug': repo.slug, 'state_slug': state.slug}), [state], state)
            )
        template_values['tab_infos'] = tab_infos
        return render_to_response('repositories/pypi/index.html', template_values, RequestContext(request))

    def search_plugin(self, request, rid, repo_slug):
        for k, v in request.META.items():
            if k.startswith('HTTP'):
                print(k, v)
        r = requests.get('https://plugins.jetbrains.com/?build=PY-141.1245')
        print(r.text)
        return HttpResponse(r.text)

    @staticmethod
    def get_filename(request, eid):
        from moneta.views import get_file

        return get_file(request, eid)

    # noinspection PyUnusedLocal
    def simple(self, request, rid, repo_slug, state_slug=None, search_pattern=''):
        search_pattern = search_pattern.replace('-', '').replace('_', '')
        repo = get_object_or_404(Repository.reader_queryset(request), id=rid, archive_type=self.archive_type)
        base_query = Element.objects.filter(repository=repo)
        if state_slug:
            state = get_object_or_404(ArchiveState, repository=repo, name=state_slug)
            base_query = base_query.filter(states=state)
        if search_pattern:
            base_query = base_query.filter(archive__iexact=search_pattern)
        view_name = moneta_url(repo, 'get_file')
        elements = [(x.filename, x.md5, reverse(view_name, kwargs={'eid': x.id, })) for x in base_query[0:1000]]
        template_values = {'elements': elements, 'rid': rid, }
        return render_to_response('repositories/pypi/simple.html', template_values, RequestContext(request))



# coding=utf-8
""" Django settings for Moneta project. """
from django.utils.six import u

__author__ = u('flanker')

import os
from os.path import join, dirname, abspath
CONF_IS_SET = False
# define a root path for misc. Django data (SQLite database, static files, ...)
LOCAL_PATH = abspath(join(dirname(dirname(dirname(__file__))), 'django_data'))

# TO BE CONFIGURED
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',  # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': join(LOCAL_PATH, 'database.sqlite3'),  # Or path to database file if using sqlite3.
        # The following settings are not used with sqlite3:
        'USER': '',
        'PASSWORD': '',
        'HOST': '',  # Empty for localhost through domain sockets or '127.0.0.1' for localhost through TCP.
        'PORT': '',  # Set to empty string for default.
    },
}

DATABASE_ROUTERS = ['moneta.core.routers.BaseRouter', ]

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts

SITE_ID = 1
USE_I18N = True
USE_L10N = True
USE_TZ = True
MEDIA_URL = '/media/'
STATIC_URL = '/static/'
STATICFILES_DIRS = (
    abspath(join(dirname(__file__), 'static')),
)
STATICFILES_FINDERS = ['django.contrib.staticfiles.finders.AppDirectoriesFinder',
                       'django.contrib.staticfiles.finders.FileSystemFinder', ]
TEMPLATE_LOADERS = [
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
]

WSGI_APPLICATION = 'moneta.core.wsgi.application'
TEMPLATE_DIRS = (
    abspath(join(dirname(__file__), 'templates')),
)
GRAPPELLI_ADMIN_TITLE = 'Administration'
ROOT_URLCONF = 'moneta.core.root_urls'
INTERNAL_IPS = ('127.0.0.1', )
DEBUG_TOOLBAR_PANELS = [
    'debug_toolbar.panels.versions.VersionsPanel',
    'debug_toolbar.panels.timer.TimerPanel',
    'debug_toolbar.panels.settings.SettingsPanel',
    'debug_toolbar.panels.headers.HeadersPanel',
    'debug_toolbar.panels.request.RequestPanel',
    'debug_toolbar.panels.sql.SQLPanel',
    'debug_toolbar.panels.staticfiles.StaticFilesPanel',
    'debug_toolbar.panels.templates.TemplatesPanel',
    'debug_toolbar.panels.cache.CachePanel',
    'debug_toolbar.panels.signals.SignalsPanel',
    'debug_toolbar.panels.redirects.RedirectsPanel',
]
DEBUG_TOOLBAR_PATCH_SETTINGS = False
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'django.contrib.sites',
    'django.contrib.sitemaps',
    'django.contrib.admin',
    'bootstrap3',
    'moneta.core',
    'moneta.repository',
    'moneta.repositories',
    'grappelli',
]

BOOTSTRAP3 = {
    'jquery_url': STATIC_URL + 'js/jquery.min.js',
    'base_url': STATIC_URL + 'bootstrap3/',
    'css_url': None,
    'theme_url': None,
    'javascript_url': None,
    'horizontal_label_class': 'col-md-2',
    'horizontal_field_class': 'col-md-4',
}


SECRET_KEY = 'kg0ohOjT3WxuYejDf4Q0VPzxldt4Q3FzmEaoMskYH5OyVxoq4X'
MIDDLEWARE_CLASSES = [
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'moneta.core.middleware.MonetaRemoteUserMiddleware',
    'moneta.core.middleware.FakeAuthenticationMiddleware',
    'moneta.core.middleware.BasicAuthMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'moneta.core.middleware.IEMiddleware',
    'django.middleware.cache.FetchFromCacheMiddleware',
]
TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.request',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.contrib.messages.context_processors.messages',
    'moneta.core.context_processors.context_base',
)
AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.RemoteUserBackend',
    'django.contrib.auth.backends.ModelBackend',
]
# TO BE CONFIGURED
CACHES = {'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache', }, }

# TO BE CONFIGURED
DEBUG = True
# TO BE CONFIGURED
TEMPLATE_DEBUG = DEBUG

# TO BE CONFIGURED
ADMINS = (("flanker", "flanker@19pouces.net"), )
# TO BE CONFIGURED
MANAGERS = ADMINS

# TO BE CONFIGURED
MEDIA_ROOT = join(LOCAL_PATH, 'media')
# TO BE CONFIGURED
STATIC_ROOT = join(LOCAL_PATH, 'static')
UPLOAD_ROOT = 'uploads'

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
# TO BE CONFIGURED
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        'moneta.files': {
            'handlers': ['mail_admins'],
            'level': 'INFO',
            'propagate': True,

        }
    }
}

# TO BE CONFIGURED

# TO BE CONFIGURED
GNUPG_HOME = os.path.join(LOCAL_PATH, 'gpg')
# TO BE CONFIGURED
TEMP_ROOT = os.path.join(LOCAL_PATH, 'tmp')
# TO BE CONFIGURED
GNUPG_KEYID = '64BCA36C3C47B697'
# TO BE CONFIGURED
GNUPG_PATH = '/usr/local/Cellar/gnupg/1.4.18/bin/gpg'

# TO BE CONFIGURED
ALLOWED_HOSTS = []
# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
# TO BE CONFIGURED
TIME_ZONE = 'Europe/Paris'
# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
# TO BE CONFIGURED
LANGUAGE_CODE = 'fr-fr'
# TO BE CONFIGURED
SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
# TO BE CONFIGURED
USE_X_FORWARDED_HOST = True
# TO BE CONFIGURED
AUTHENTICATION_HEADER = 'REMOTE_USER'
# TO BE CONFIGURED
FAKE_AUTHENTICATION_USERNAME = 'flanker'
TEST_RUNNER = 'django.test.runner.DiscoverRunner'
# TO BE CONFIGURED  => use X-Sendfile header, header['X-Sendfile'] = the absolute_path of the file to send
USE_X_SEND_FILE = False
# TO BE CONFIGURED  => use X-Accel-Redirect header, = X_ACCEL_REDIRECT_ARCHIVE + relative path to send
X_ACCEL_REDIRECT_ARCHIVE = None
# TO BE CONFIGURED  => use X-Accel-Redirect header, = X_ACCEL_REDIRECT_UNCOMPRESSED + relative path to send
X_ACCEL_REDIRECT_UNCOMPRESSED = None
# TO BE CONFIGURED  => use X-Accel-Redirect header, = X_ACCEL_REDIRECT_CACHE + relative path to send
X_ACCEL_REDIRECT_CACHE = None
# TO BE CONFIGURED
EMAIL_HOST = 'localhost'
# TO BE CONFIGURED
EMAIL_HOST_PASSWORD = ''
# TO BE CONFIGURED
EMAIL_HOST_USER = ''
# TO BE CONFIGURED
EMAIL_PORT = 25
# TO BE CONFIGURED
EMAIL_SUBJECT_PREFIX = '[Moneta]'
# TO BE CONFIGURED
EMAIL_USE_TLS = False
# TO BE CONFIGURED
EMAIL_USE_SSL = False
# TO BE CONFIGURED
SERVER_EMAIL = 'root@localhost'
